use log::LevelFilter;

use log4rs::{
    append::{
        console::ConsoleAppender,
        rolling_file::{
            policy::compound::{
                roll::fixed_window::FixedWindowRoller, trigger::size::SizeTrigger, CompoundPolicy,
            },
            RollingFileAppender,
        },
    },
    config::{Appender, Logger, Root},
    encode::pattern::PatternEncoder,
    Config,
};

const FILE_MAX_SIZE: u64 = 30000000; //30 mB
const LOGS_MAX_FILES: u32 = 10;

const LOG_FORMAT: &str = "{d(%Y-%m-%d %H:%M:%S)} | {({l}):5.5} | {f}:{L} — {m}{n}";
const LOGS_FILE: &str = "logs/yorokobot_latest.log";
const LOGS_ARCHIVE_FILE_PATTERN: &str = "logs/yorokobot_{}.gz";

/// Configure the bot logger
pub fn init_logger() -> log4rs::Handle {
    // Rollings logs trigger
    let size_triger = SizeTrigger::new(FILE_MAX_SIZE);

    // Rolling logs rollers
    let logs_roller = FixedWindowRoller::builder()
        .base(1)
        .build(LOGS_ARCHIVE_FILE_PATTERN, LOGS_MAX_FILES)
        .unwrap();

    // Rolling logs policies
    let rolling_logs_policy = CompoundPolicy::new(Box::new(size_triger), Box::new(logs_roller));

    // Appenders
    let stdout = ConsoleAppender::builder()
        .encoder(Box::new(PatternEncoder::new(LOG_FORMAT)))
        .build();
    let rolling_logs = RollingFileAppender::builder()
        .encoder(Box::new(PatternEncoder::new(LOG_FORMAT)))
        .build(LOGS_FILE, Box::new(rolling_logs_policy))
        .unwrap();

    let config = Config::builder()
        .appender(Appender::builder().build("stdout", Box::new(stdout)))
        .appender(Appender::builder().build("rolling_logs", Box::new(rolling_logs)))
        // Don't print the serenity crate logs
        .logger(Logger::builder().build("serenity", LevelFilter::Error))
        // Don't print the tracing crate logs
        .logger(Logger::builder().build("tracing", LevelFilter::Error))
        .build(
            Root::builder()
                .appender("stdout")
                .appender("rolling_logs")
                .build(LevelFilter::Info),
        )
        .unwrap();

    log4rs::init_config(config).unwrap()
}
